@@ -30,7 +30,7 @@ use URI;
 
 @ISA = qw(Exporter AutoLoader XTM::generic);
 @EXPORT = qw( );
-$VERSION = '0.10';
+$VERSION = '0.11';
 
 =pod
 
@@ -51,7 +51,7 @@ XTM::topic - Topic Map management, Topic
   my $t2 = new XTM::topic (id => '1234');
 
   # there is also a cheap way to populate the topic with a default
-  my $t3 = new XTM::topic (id => '1234', populate => XTM::topic::default);
+  my $t3 = new XTM::topic (id => '1234', populate => \&XTM::topic::default_populate);
 
 
   # see XTM::generic for more methods
@@ -68,11 +68,30 @@ The constructor expects a hash with following (optional) fields:
 
 =over
 
-=item id: a topic id (unique in the map in use), if not given, it will be generated
+=item C<id>:
+
+a topic id (unique in the map in use), if not given, it will be generated
+
+=item C<populate>:
+
+a code reference to a subroutine populating the topic. There are the following B<predefined>
+subroutines:
+
+=over
+
+=item I<default_populate>
+
+This routine expects as first parameter a C<XTM::topic>-blessed object and fills in
+the PSI topic as 'instanceOf', global scope where appropriate. It also generates in a fairly
+ad-hoc manner a 'baseName' from the topic id (substituting '-' by ' ').
 
 =back
 
-  $t = new XTM::topic ('id' => '#123');
+
+=back
+Example:
+
+  $t = new XTM::topic ('id' => 'x123');
 
 =cut
 
@@ -81,28 +100,37 @@ my $topic_cntr = 0;
 sub new {
   my $class   = shift;
   my %options = @_;
+
+  if ($options{id}) {
+    unless ($options{id} =~ /^[\w_:][\w\d\-\.]*/ && $options{id} !~ /^xml:/) {  # Professional XML, page 33
+      elog ($class, 1, "topic ID '$options{id}' might make problems with XML processors");
+    }
+  } else {
+    $options{id} = sprintf ("t-%10.10d", $topic_cntr++);
+  }
   elog ($class, 5, "new '$options{id}'");
 
-  elog ($class, 1, "topic ID '$options{id}' might make problems with XML processors")
-    unless $options{id} =~ /^[\w_:][\w\d\-\.]*/ && $options{id} !~ /^xml:/;  # Professional XML, page 33
-
-
-  my $self  = bless { id          => $options{id} || sprintf ("t-%10.10d", $topic_cntr++),
+  my $self  = bless { id          => $options{id},
 		      instanceOfs => [],
 		      baseNames   => [],
 		      occurrences => []
 		    }, $class;
+  $self->{ids} = [ $self->{id} ]; # eventually this will contain id and all aliases of this topic
   &{$options{populate}} ($self)
       if (defined $options{populate} && ref ($options{populate}) eq 'CODE');
   return $self;
 }
 
+#sub ids {
+#  my $self = shift;
+#  return @{$self->{ids}};
+#}
 
 # this is a simple routine (NO method) to fill a basename with a default scope
 sub default_populate {
   my $t = shift;
 
-  my $name = $t->id;  # default base name
+  my $name = $t->{id};  # default base name
   $name =~ s/-/ /g;
   my $b = new XTM::baseName ();
   $b->add_baseNameString (new XTM::baseNameString (string => $name));
@@ -121,8 +149,9 @@ sub default_populate {
 
 =item I<occurrences>
 
-returns the occurrences of the topic as a list reference. If provided with a parameter, this value
-will be used as new occurrences list reference (no validation will happen).
+returns the occurrences of the topic as a list reference. If provided with a CODEREF parameter,
+this subroutine will be used to filter the occurrences. If provided with a ARRAYREF
+parameter, this will be used as new value.
 
 =cut
 
@@ -131,7 +160,11 @@ sub occurrences {
   my $sub  = shift;
   
   elog ('XTM::topic', 5, "occurrences", $sub);
-  return $sub ? [ grep (&$sub ($_), @{$self->{occurrences}}) ] : $self->{occurrences};
+  return ref ($sub) eq 'CODE' ?
+         [ grep (&$sub ($_), @{$self->{occurrences}}) ] :
+	   ref ($sub) eq 'ARRAY' ? 
+	     $self->{occurrences} = $sub :
+	       $self->{occurrences};
 }
 
 =pod
@@ -179,6 +212,73 @@ sub has_instanceOf {
 
 =pod
 
+=item I<canonicalize>
+
+This method will simplify the topic by eliminating duplicates as prescribed in F.6.
+
+   $t->canonicalize();
+
+=cut
+
+sub canonicalize {
+  my $self = shift;
+
+  if ($self->instanceOfs) {
+    my $p = 0;
+    $self->instanceOfs ([ grep ($_->reference->href ne $p && ($p = $_->reference->href), 
+                           sort {$a->reference->href cmp $b->reference->href} @{$self->instanceOfs})]);
+  }
+  if ($self->baseNames) {
+    my $p = 0;
+sub bacult { # computes a value which allows to 'compare' to occurrences
+  my $bn = shift;
+  my $v;
+
+  $v .= $bn->baseNameString->string;
+  $v .= join "", map { $_->href } @{$bn->scope->references};
+  return $v;
+}
+
+    $self->baseNames ([ grep (bacult($_) ne $p && ($p = bacult($_)), 
+                           sort {bacult($a) cmp bacult($b) } @{$self->baseNames})]);
+    foreach my $bn (@{$self->baseNames}) {
+      $bn->scope->references ( [ sort {ref ($a) cmp ref ($b) || $a->href cmp $b->href } @{$bn->scope->references} ]);
+    }
+  }
+  if (0 && $self->occurrences) {
+    my $p = 0;
+sub occult { # computes a value which allows to 'compare' to occurrences
+  my $oc = shift;
+  my $v;
+
+  $v .= ref ($oc); # resourceRef or resourceData
+  if (ref($oc) eq 'XTM::resourceRef') {
+     $v .= $oc->resource->data;
+  } else {
+     $v .= $oc->resource->href;
+  }
+  $v .= join "", map { $_->href } @{$oc->scope->references};
+  $v .= $oc->instanceOf->reference->href;
+  return $v;
+}
+    $self->occurrences ([ grep (occult($_) ne $p && ($p = occult ($_)), 
+                           sort {occult ($a) cmp occult ($b) } @{$self->occurrences})]);
+    foreach my $oc (@{$self->occurrences}) {
+      $oc->scope->references ( [ sort {ref ($a) cmp ref ($b) || $a->href cmp $b->href } @{$oc->scope->references} ]);
+    }
+  }
+
+  if ($self->subjectIdentity && $self->subjectIdentity->references) {
+    my $p = 0;
+    $self->subjectIdentity->references ([
+         grep ($_->href ne $p && ($p = $_->href),
+               sort {$a->href cmp $b->href} @{$self->subjectIdentity->references})
+                                         ]);
+  }
+}
+
+=pod
+
 =item I<connected>
 
 returns a list reference of all topic references mentioned in this topic. These
@@ -260,7 +360,7 @@ L<XTM>
 
 =head1 AUTHOR INFORMATION
 
-Copyright 2001, 2002, Robert Barta <rho@telecoma.net>, All rights reserved.
+Copyright 200[1-2], Robert Barta <rho@telecoma.net>, All rights reserved.
 
 This library is free software; you can redistribute it
 and/or modify it under the same terms as Perl itself.
